/*************************************************************************
 *
 *   Program:  NRF24L01_Processor 				Version: 1.00
 *
 *   Author:  SS								Date:  Dec 19, 2014
 *
 *   This program implements functions to control a NRF24L01 wireless peripheral attached to the Nano board.
 *   The NRF24L01 is a high speed wireless module running at 2.4G with 127 channels, 5 byte address setting
 *   per channel and 32 byte packet payload size.  Functions are provided to allow the NRF24L01 to operate 
 *   as a sender, receiver or transceiver.  An enumeration functions checks to see how many devices are in 
 *   as specific channel setting.  The idea is to provide all the necessary low level calls to create a multi-
 *   node wireless network (although no Mesh network is provided).
 *
 *   This project is licensed under the MIT license.  You can modify, enhance, etc anyway you like.
 *
 *   SOC Robotics, Inc is not liable in anyway for the use of this software and assumes no responsibility 
 *   for it's use.  No warranties of any kind are provided.
 *
 *   Copyright 2014, SOC Robotics, Inc.  
 *   There maybe code from other parties in this application  If we don't mention them by name thank you
 *   and if we should please let us know and we will include your name and attribution.
 *
 *************************************************************************/
//#include "hw_config.h"

#include "stdint.h"
#include "stdbool.h"
#include "stdio.h"
#include "math.h"
#include "string.h"
#include "stddef.h"
#include "stdlib.h"
#include "ctype.h"

#include "NRF24L01.h"
#include "FXOS8700.h"
#include "PicoM_Defines.h"
#include "PicoM_Externals.h"
#include "colors.h"

#define LED1ON  GPIO_ResetBits(GPIOB,GPIO_Pin_0);

extern volatile struct sid {
	unsigned int rawtemperature,rawhumidity;
	float temperature,humidity;
} si7013;

extern FXOS_F dip[];
extern FXOS_F pip[];
extern FXOS_F mcp[];

/***************************************************
  RF24L01 Functions
***************************************************/

volatile uint8_t TADDR[5] = {'S', 'O', 'C', DEVICE_CLASS, DEVICE_ID}; // device address
volatile unsigned char payload[32],rf_state,rf_mode,device_id,rf_payload_size,channel_no,rf_device_class;
volatile bool rf_inmessage;

/*!
 * \brief Initializes the transceiver.
 */
void RF24_Init(void) 
{
	RF_CE_LOW();   // CE low: do not send or receive data
	RF_CSN_HIGH(); // CSN high: not sending commands to the device
}

/*!
* \brief Writes a byte and reads the value
* \param val Value to write. This value will be shifted out
* \return The value shifted in
*/
volatile static uint8_t SPI_WriteRead(volatile uint8_t val) 
{
	return SPI2_transfer_byte(val);
}

/*!
 * \brief Write a register value to the transceiver
 * \param reg Register to write
 * \param val Value of the register to write
 */
void RF_WriteRegister(volatile uint8_t reg, volatile uint8_t val) 
{	
	//printf("writeregister:%x %x\n\r",reg,val);
	RF_CSN_LOW(); // initiate command sequence
	//Puts("write first byte register\n\r");
	SPI2_transfer_byte(RF24_W_REGISTER|reg);
	//Puts("write second byte register\n\r");
	SPI2_transfer_byte(val);

	RF_CSN_HIGH(); // end command sequence
	delay_us(10); // insert a delay until next command
	//Puts("Leaving WriteRegister function\n\r");
}

unsigned char RF_ReadRegister(volatile uint8_t reg)
{
	volatile unsigned char inchar;
	
	RF_CSN_LOW(); 
	SPI2_transfer_byte(RF24_R_REGISTER|reg);
	inchar = SPI2_transfer_byte(0xFF);
	RF_CSN_HIGH();
	delay_us(10);
	return inchar;
}

// Assumes CSN pulled low
void RF_Write(volatile unsigned char data)
{
	SPI2_transfer_byte(data);
}

/*!
* \brief Read and return the STATUS
* \return Status
*/
volatile uint8_t RF_GetStatus(void) 
{
	volatile unsigned char status;
	
	RF_CSN_LOW();
	status = SPI_WriteRead(RF24_R_REGISTER|RF24_STATUS);
	RF_CSN_HIGH();
	
	return status;
}

/*!
* \brief Read multiple bytes from the bus.
* \param reg Register address
* \param buf Buffer where to write the data
* \param bufSize Buffer size in bytes
*/
void RF_ReadRegisterData(volatile uint8_t reg, uint8_t *buf, volatile uint8_t bufSize)
{
	volatile int i;
	
	RF_CSN_LOW();
	SPI_WriteRead(RF24_R_REGISTER|reg);
	
	for(i=0; i<bufSize; i++) buf[i] = SPI_WriteRead(0xff) ;
	
	RF_CSN_HIGH();
	delay_us(10);
}

/*!
* \brief Write multiple bytes from the bus.
* \param reg Register address
* \param buf Buffer where to write the data
* \param bufSize Buffer size in bytes
*/
void RF_WriteRegisterData(volatile uint8_t reg, uint8_t *buf, volatile uint8_t bufSize)
{
	volatile int i;
	
	RF_CSN_LOW();
	SPI_WriteRead(RF24_W_REGISTER|reg);
	
	for(i=0; i<bufSize; i++) {
		SPI_WriteRead(buf[i]);
		//if(buf[i]!=0) printf("%c",buf[i]);
		}
	
	RF_CSN_HIGH();
	delay_us(10);
}

/*
** ===================================================================
**     Event       :  IRQ_OnInterrupt (module Events)
**
**     Component   :  IRQ [ExtInt]
**     Description :
**         This event is called when an active signal edge/level has
**         occurred.
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
static volatile bool isrFlag; // flag set by ISR

void APP_OnInterrupt(void)
{
	//CE_ClrVal(); // stop sending/listening
	isrFlag = true;
}

void RF24_Payload_Channel(volatile unsigned char payloadsize,volatile unsigned char channel)
{
	rf_payload_size = payloadsize;
	RF_WriteRegister(RF24_RX_PW_P0, payloadsize); // number of payload bytes we want to send and receive
	channel_no = channel;
	RF_WriteRegister(RF24_RF_CH, channel); // set channel
}

void RF24L01_Processor_Init(volatile char rftype,volatile uint8_t payloadsize,volatile uint8_t channel)
{	
	RF24_Init(); // set CE and CSN to initialization value
	//RF_WriteRegister(RF24_RF_SETUP, RF24_RF_SETUP_RF_PWR_0|RF24_RF_SETUP_RF_DR_250);
	//RF_WriteRegister(RF24_RF_SETUP, RF24_RF_SETUP_RF_PWR_0|RF24_RF_SETUP_LNA_HCURR|RF24_RF_SETUP_RF_DR_1000);
	
	//Puts("Call rf24 payload\n\r");

	RF24_Payload_Channel(payloadsize,channel);

	// Set RADDR and TADDR as the transmit address since we also enable auto acknowledgment
	TADDR[4] = device_id;
	RF_WriteRegisterData(RF24_RX_ADDR_P0, (uint8_t*)TADDR, 5);
	RF_WriteRegisterData(RF24_TX_ADDR, (uint8_t*)TADDR, 5);

	// Enable RX_ADDR_P0 address matching
	//RF_WriteRegister(RF24_EN_RXADDR, RF24_EN_RXADDR_ERX_P0); // enable data pipe 0
	
	RF_WriteRegister(RF24_SETUP_RETR, RF24_SETUP_RETR_ARD_750|RF24_SETUP_RETR_ARC_6); // Important: need 750 us delay between every retry

	if(rftype==RFSENDER) {
		//printf("RF24L01 sender enabled\n\r");
		//RF_WriteRegister(RF24_EN_AA, RF24_EN_AA_ENAA_P0); // enable auto acknowledge. RX_ADDR_P0 needs to be equal to TX_ADDR!
		//RF_WriteRegister(RF24_SETUP_RETR, RF24_SETUP_RETR_ARD_750|RF24_SETUP_RETR_ARC_0); // Important: need 750 us delay - no retry
		
		TX_POWERUP();  // Power up in transmitting mode
		delay_us(150);
		//CE_ClrVal();   // Will pulse this later to send data
		}
	
	else {
		//printf("RF24L01 receiver enabled\n\r");
		
		RX_POWERUP();  // Power up in receiving mode
		delay_us(150);
		//CE_SetVal();   // Listening for packets
		}
}


void IRQ_OnInterrupt(void)
{
	APP_OnInterrupt();
}

/*!
* \brief Reset the given mask of status bits
* \param flags Flags, one or more of RF24_STATUS_RX_DR, RF24_STATUS_TX_DS, RF24_STATUS_MAX_RT
*/
void RF_ResetStatusIRQ(volatile uint8_t flags) 
{
	delay_us(10);
	RF_CSN_LOW();
	delay_us(10);
	RF_WriteRegister(RF24_STATUS, flags); // reset all IRQ in status register
	delay_us(10);
	RF_CSN_HIGH();
	delay_us(10);
}

/*!
* \brief Send the payload to the Tx FIFO and send it
* \param payload Buffer with payload to send
* \param payloadSize Size of payload buffer
*/
void RF_TxPayload(uint8_t *payload, volatile uint8_t payloadSize)
{
	//RF_WriteRegister(RF24_CONFIG, RF24_PWR_UP|RF24_PRIM_TX); 
	
	RF_CSN_LOW();
	RF_Write(RF24_FLUSH_TX); 
	RF_CSN_HIGH();
	
	RF_WriteRegisterData(RF24_W_TX_PAYLOAD, payload, payloadSize); 

	RF_CE_HIGH(); 
	delay_us(15); 
	//delay_us(70);
	RF_CE_LOW();  
}

/*!
 * \brief Receive the Rx payload from the FIFO and stores it in a buffer.
 * \param payload Pointer to the payload buffer
 * \param payloadSize Size of the payload buffer
 */
void RF_RxPayload(uint8_t *payload, volatile uint8_t payloadSize) 
{
	RF_ReadRegisterData(RF24_R_RX_PAYLOAD, payload, payloadSize); 
}

// Polled send
void RF_SendDataCh(unsigned char *payload,volatile uint8_t payloadsize,volatile uint8_t channel)
{
	RF_TxPayload(payload,payloadsize);
}

// Polled send
void RF_SendData(unsigned char *payload,volatile uint8_t payloadsize)
{
	RF_TxPayload(payload,payloadsize);
	
	if(RF_GetStatus()&RF24_STATUS_MAX_RT) {
		RF_WriteRegister(RF24_STATUS,RF24_STATUS_MAX_RT);
		}
}

// Switch to tx mode from rx mode
void RF_SendDataCheck(unsigned char *payload,volatile uint8_t payloadsize)
{
	RF_CE_LOW();
	RXTX_POWERDOWN();
	TX_POWERUP();
	delay_us(1000);  // Reduce this number so it works faster with Nano processor
	RF_SendData(payload,payloadsize);
	delay_us(750);  // Allow packet time to transmit - including retries - convert to status check
					// TBD - make this smart so channel is free'd as quickly as possible
	RXTX_POWERDOWN();
	RX_POWERUP();
	delay_us(150);
	RF_CE_HIGH(); 
	delay_us(10);
}

// Switch to tx mode from rx mode
void RF_SendFXOSTransmitCheck(void)
{
	RF_CE_LOW();
	RXTX_POWERDOWN();
	TX_POWERUP();
	delay_us(1000);  // Reduce this number so it works faster with Nano processor
}

// Switch to tx mode from rx mode
void RF_SendFXOSRxCheck(void)
{
	RXTX_POWERDOWN();
	RX_POWERUP();
	delay_us(150);
	RF_CE_HIGH();
	delay_us(10);
}

int RF_SendData_Int(void)
{
	volatile int status;
	
	// Sender routine in main
	if (isrFlag) { // check if we have received an interrupt
		isrFlag = false; // reset interrupt flag
		status = RF_GetStatus();
		if (status&RF24_STATUS_RX_DR) { // data received interrupt
			RF_ResetStatusIRQ(RF24_STATUS_RX_DR); // clear bit
			}
		if (status&RF24_STATUS_TX_DS) { // data sent interrupt
			GreenLedOFF; // indicate data has been sent
			RF_ResetStatusIRQ(RF24_STATUS_TX_DS); // clear bit
			}
		if (status&RF24_STATUS_MAX_RT) { // retry timeout interrupt
			RF_ResetStatusIRQ(RF24_STATUS_MAX_RT); // clear bit
			}
		}
	return 0;
}

// Non blocking polled receive
int RF_GetData(void)
{
	volatile unsigned char status;
	
	status = RF_GetStatus();
	
	//printf("status=%x  \r",rfstatus);
	if(status&RF24_STATUS_RX_DR) { // data received interrupt
		RF_RxPayload(payload, rf_payload_size); // will reset RX_DR bit
		RF_ResetStatusIRQ(RF24_STATUS_RX_DR|RF24_STATUS_TX_DS|RF24_STATUS_MAX_RT); // make sure we reset all flags. Need to have the pipe number too
		return rf_payload_size;
		}
	if(status&RF24_STATUS_MAX_RT) { // retry timeout interrupt
		RF_ResetStatusIRQ(RF24_STATUS_MAX_RT); // clear bit
		return -2;
		}
	return -1;
}

// Non blocking polled receive
int RF_GetDataWait(volatile int waittime)
{
	volatile unsigned char status;

	waittime = waittime * 40;
	//delay_ms(10);
	//Start_TCC0();
	//do {
		status = RF_GetStatus();
		if(status&RF24_STATUS_RX_DR) { // data received interrupt
			RF_RxPayload(payload, sizeof(payload)); // will reset RX_DR bit
			RF_ResetStatusIRQ(RF24_STATUS_RX_DR|RF24_STATUS_TX_DS|RF24_STATUS_MAX_RT); // make sure we reset all flags. Need to have the pipe number too
			return sizeof(payload);
			}
		if(status&RF24_STATUS_MAX_RT) { // retry timeout interrupt
			RF_ResetStatusIRQ(RF24_STATUS_MAX_RT); // clear bit
			return -2;
			}
	//	}
	//while(1<waittime);
	//while(TCD0.CNT<waittime);
	//Stop_TCC0();
	return -1;
}

// Switch to RX mode before getting data then switch back
int RF_GetDataWaitCheck(volatile int waittime)
{
	volatile int payloadbytes;
	
	RXTX_POWERDOWN();
	RX_POWERUP();
	RF_CE_HIGH(); 
	delay_us(150);
	payloadbytes = RF_GetDataWait(waittime);
	RF_CE_LOW(); 
	RXTX_POWERDOWN();
	TX_POWERUP();
	delay_us(150);
	return payloadbytes;
}

void RF_DumpRegisters(int mode)
{
	unsigned char buf[5];
	int i;
/*
	if(mode==1) {
		printf("\n\rRF24_CONFIG=%x\n\r",RF_ReadRegister(RF24_CONFIG));
		printf("RF24_EN_AA=%x\n\r",RF_ReadRegister(RF24_EN_AA));
		printf("RF24_EN_RXADDR=%x\n\r",RF_ReadRegister(RF24_EN_RXADDR));
		printf("RF24_SETUP_AW=%x\n\r",RF_ReadRegister(RF24_SETUP_AW));
		printf("RF24_SETUP_RETR=%x\n\r",RF_ReadRegister(RF24_SETUP_RETR));
		printf("RF24_RF_SETUP=%x\n\r",RF_ReadRegister(RF24_RF_SETUP));
		printf("RF24_STATUS=%x\n\r",RF_ReadRegister(RF24_STATUS));
		printf("RF24_OBSERVE_TX=%x\n\r",RF_ReadRegister(RF24_OBSERVE_TX));
		printf("RF24_RPD=%x\n\r",RF_ReadRegister(RF24_RPD));
		printf("RF24_RX_PW_P0=%x\n\r",RF_ReadRegister(RF24_RX_PW_P0));
		printf("RF24_FIFO_STATUS=%x\n\r",RF_ReadRegister(RF24_FIFO_STATUS));
		}
	
	printf("RF24_RF_CH=%x\n\r",RF_ReadRegister(RF24_RF_CH));
	printf("RF24_RX_ADDR_P0= ");
	RF_ReadRegisterData(RF24_RX_ADDR_P0, buf, 5);
	for(i=0; i<5; i++) printf("%x ",buf[i]);
	for(i=0; i<4; i++) PutChar(buf[i]);
	printf(" %x ",buf[i]);
	printf("\n\r");
	
	printf("RF24_TX_ADDR   = ");
	RF_ReadRegisterData(RF24_RX_ADDR_P0, buf, 5);
	for(i=0; i<5; i++) printf("%x ",buf[i]);
	for(i=0; i<4; i++) PutChar(buf[i]);
	printf(" %x ",buf[i]);
	printf("\n\r");
*/
}

void RF24L01_Test(void)
{
	volatile char inchar,outstrg[17];
	volatile unsigned char status,i;
	volatile int payloadbytes,loopcnt=0,loopcnt2=0;
	
	Puts("\n\rRF24L01 Test A\n\r");
	Puts("Enter t-transmit, r-receive, e-exit: ");
	//putchar(inchar=getchar());
	inchar = 't';
	GreenLedOFF;
	if(inchar=='t')  RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
	else RF24L01_Processor_Init(RFRECEIVER,rf_payload_size,channel_no);
	RF_DumpRegisters(0);
	GreenLedON;
	if(inchar=='t') {
		//printf("\n\rTransmit string: Hello world\n\r");
		while((inchar=GetCharnw3())!='e') {
			//StartTimer();
			GreenLedON;
			//sprintf(outstrg,"Hello world %4d",loopcnt++);
			RF_SendData(outstrg,16);
			//printf("Sent=%s   ",outstrg);

			delay_ms(1);
			status = RF_GetStatus();
			//printf("%x \r",status);
			payloadbytes = RF_GetDataWaitCheck(100);
			payloadbytes = 0;
			if(payloadbytes>0) {
				//printf("remote payload=");
				//for(i=0; i<payloadbytes; i++)  printf("%c",payload[i]);
				}
			//printf("\n\r");
			//etime = StopTimer();
			//printf(" et=%f\n\r",etime);
			delay_ms(150);
			GreenLedOFF;
			delay_ms(150);
			}
		}
	else if(inchar=='r') {
		Puts("\n\rReceive string:\n\r");
		RF_CE_HIGH(); // start transmission
		delay_us(150);
		//StartTimer();
		while((inchar=getchar())!='e') {
			status = RF_GetStatus();
			payloadbytes = RF_GetData();

			//printf("s=%x pl=%d ",status,payloadbytes);
			if(payloadbytes>0) {
				//for(i=0; i<payloadbytes; i++)  printf("%c",payload[i]);
				Puts("\n\r");

				//delay_ms(1);
				//sprintf(payload,"return pack %4d",loopcnt2++);
				//RF_Process_Payload(payload);
				}
			else putchar('\r');
			delay_ms(10);
			
			}
		}
}

void RF24L01_ReInit(void)
{
	RF_CE_LOW();
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFRECEIVER,rf_payload_size,channel_no);
	RF_CE_HIGH();
	delay_us(150);
}

// Check if RF24L01 is present and initialize if it is
void rf24L01_Init(volatile char rftype)
{	
	//Puts("\n\rin rf24l01 init\n\r");

	// Enable with default parameters as sender or receiver
	RF24L01_Processor_Init(rftype,rf_payload_size,channel_no);

	// Check if rf24L01 is present by verifying contents of channel number
	// and set default rx or tx state
	if(RF_ReadRegister(RF24_RF_CH)==channel_no)  {
		if(rftype==RFSENDER)  rf_state = RF_STATE_INITIALIZED_TX;
		else {
			RF_CE_HIGH();  // fire up receiver
			rf_state = RF_STATE_INITIALIZED_RX;
			}
		}
	else rf_state = RF_STATE_NOTPRESENT;
}

void ZeroPayload(void)
{
	volatile int i;
	
	for(i=0; i<PAYLOAD_SIZE; i++) payload[i] = 0;
}

void Display_RF_Status(void)
{
	char outstrg[35];
	
	if(rf_state==RF_STATE_NOTPRESENT) {
		Puts("RF24L01 Not Active");
#ifdef ST7735LCD
		ST7735_drawString(1, 30,"RF24L01 Not Active",colorsRGB24toRGB565(0,0,0), 1);
#endif
#ifdef SSD1306OLED
	ssd1306_printf(1,0,"RF24L01 InActive");
#endif
		}
	else {
		if(rf_state==RF_STATE_INITIALIZED_RX) {
			Puts("RF24L01 RX Mode Active, ");
#ifdef ST7735LCD
			sprintf(outstrg,"RF24L01 RX Mode Live, ");
			ST7735_drawString(1, 30,outstrg,colorsRGB24toRGB565(0,0,0), 1);
#endif
#ifdef SSD1306OLED
			ssd1306_printf(1,0,"RX Live On");
#endif
			}
		else if(rf_state==RF_STATE_INITIALIZED_TX) {
			Puts("RF24L01 TX Mode Active, ");
#ifdef ST7735LCD
			sprintf(outstrg,"RF24L01 TX Mode Live, ");
			ST7735_drawString(1, 30,outstrg,colorsRGB24toRGB565(0,0,0), 1);
#endif
#ifdef SSD1306OLED
			ssd1306_printf(1,0,"TX Live On");
#endif
			}
		Puts("Ch=");
		PrintInt(channel_no);
		Puts(" ID=");
		PrintInt(device_id);
		Puts(" DC=");
		PutChar(rf_device_class);

#ifdef ST7735LCD
		sprintf(outstrg,"On Ch=%d ID=%d DC=%c",channel_no,device_id,rf_device_class);
		ST7735_drawString(1, 41,outstrg,colorsRGB24toRGB565(0,0,0), 1);
#endif
#ifdef SSD1306OLED
			ssd1306_printf(2,0,"Ch=%d ID=%d DC=%c",channel_no,device_id,rf_device_class);
#endif
		}
}

extern volatile struct imudata {
	int ai[3];
	int gi[3];
	int mi[3];
	int temp;
	float af[3];
	float gf[3];
	float mf[3];
	int tempi;
	int rolli;
	int pitchi;
	int yawi;
	int pressi;
	float tempf;
	float roll;
	float pitch;
	float yaw;
	float press;
	char mget;
} imu;

void IMU8420_Init(void)
{
	int i;

	union {
		unsigned char buf[32];
		struct rfmemsdata {
			unsigned long time;
			short int accel[3];
			short int gyro[3];
			short int mag[3];
			short int roll,pitch,yaw;
			short int temp,press;
		} data;
	} rf;

	rf.data.time = 1234;
	for(i=0; i<3; i++) {
		rf.data.accel[i] = 100+i;
		rf.data.gyro[i] = 200+i;
		rf.data.mag[i] = 300+i;
		}
	rf.data.roll = 1220;
	rf.data.pitch = 2340;
	rf.data.yaw = 3450;
	rf.data.temp = 2300;
	rf.data.press = 14300;

	for(i=0; i<32; i++)  {
		imu8420buf[i] = rf.buf[i];
		//PrintHexByte(imu8420buf[i]); PutChar(' ');
		}
}

// Diagnostic dump of MEMs and EKF data for RF transfer
void PrintfRF(unsigned char *rfbuf)
{
	int i,hours,minutes;
	volatile float seconds,accelscale,gyroscale,magscale;
	volatile float fnum,fnum2;

	union {
		unsigned char buf[32];
		struct rfmemsdata {
			unsigned long time;
			short int accelx,accely,accelz;
			short int gyrox,gyroy,gyroz;
			short int magx,magy,magz;
			short int roll,pitch,yaw;
			short int temp,press;
		} data;
	} rf;

	accelscale = 1.0;
	gyroscale = 1.0;
	magscale = 1.0;

	//Puts("\n\r");
	for(i=0; i<32; i++) {
		rf.buf[i] = rfbuf[i];
		//PrintInt(rf.buf[i]); PutChar(' ');
		//PrintHexByte(rf.buf[i]); PutChar(' ');
		}
	//Puts("\n\r");
	if(0) {
		//printf("%ld ",rf.data.time);
		PrintInt(rf.data.time); PutChar(' ');

		printf("%d %d %d ",rf.data.accelx,rf.data.accely,rf.data.accelz);
		PrintInt(rf.data.accelx); PutChar(' ');
		PrintInt(rf.data.accely); PutChar(' ');
		PrintInt(rf.data.accelz); PutChar(' ');

		printf("%d %d %d ",rf.data.gyrox,rf.data.gyroy,rf.data.gyroz);
		PrintInt(rf.data.gyrox); PutChar(' ');
		PrintInt(rf.data.gyroy); PutChar(' ');
		PrintInt(rf.data.gyroz); PutChar(' ');

		printf("%d %d %d ",rf.data.magx,rf.data.magy,rf.data.magz);
		PrintInt(rf.data.magx); PutChar(' ');
		PrintInt(rf.data.magy); PutChar(' ');
		PrintInt(rf.data.magz); PutChar(' ');

		printf("%d %d %d ",rf.data.roll,rf.data.pitch,rf.data.yaw);
		PrintInt(rf.data.roll); PutChar(' ');
		PrintInt(rf.data.pitch); PutChar(' ');
		PrintInt(rf.data.yaw); PutChar(' ');

		printf("%d %d\n\r",rf.data.temp,rf.data.press);
		PrintInt(rf.data.temp); PutChar(' ');
		PrintInt(rf.data.press); Puts("\n\r$");
		}
	else {
		Puts("\n\rDump of IMU8420 packet\n\r");
		PrintInt(rf.data.time); PutChar(' ');

		hours = rf.data.time/(3600*1000);
		rf.data.time = rf.data.time - (hours*3600*1000);
		minutes = rf.data.time/(60*1000);
		rf.data.time = rf.data.time - (minutes*60*1000);
		seconds = ((float) rf.data.time)/1000.0;
		//printf("%02d:%02d:%6.4f ",hours,minutes,seconds);
		PrintInt(hours); PutChar(':'); PrintInt(minutes); PutChar(':'); Printftoa(seconds);Puts("\n\r");

		//printf("%6.3f %6.3f %6.3f ", (float)rf.data.accelx*0.0039*accelscale,(float)rf.data.accely*0.0039*accelscale,(float)rf.data.accelz*0.0039*accelscale);
		Printftoa(((float)rf.data.accelx)*0.0039*accelscale); PutChar(' ');
		Printftoa(((float)rf.data.accely)*0.0039*accelscale); PutChar(' ');
		Printftoa(((float)rf.data.accelz)*0.0039*accelscale); PutChar(' ');

		//printf("%6.1f %6.1f %6.1f ",((float) rf.data.gyrox)*250.0*gyroscale/32768.0,((float) rf.data.gyroy)*250.0*gyroscale/32768.0,((float) rf.data.gyroz)*250.0*gyroscale/32768.0);
		Printftoa(((float) rf.data.gyrox)*250.0*gyroscale/32768.0); PutChar(' ');
		Printftoa(((float) rf.data.gyroy)*250.0*gyroscale/32768.0); PutChar(' ');
		Printftoa(((float) rf.data.gyroz)*250.0*gyroscale/32768.0); PutChar(' ');

		//printf("%6.3f %6.3f %6.3f ",((float) rf.data.magx)*1.3*magscale/2048.0,((float) rf.data.magy)*1.3*magscale/2048.0,((float) rf.data.magz)*1.3*magscale/2048.0);
		Printftoa(((float) rf.data.magx)*1.3*magscale/2048.0); PutChar(' ');
		Printftoa(((float) rf.data.magy)*1.3*magscale/2048.0); PutChar(' ');
		Printftoa(((float) rf.data.magz)*1.3*magscale/2048.0); PutChar(' ');

		//printf("%6.1f %6.1f %6.1f ",(float)rf.data.roll/10.0,(float)rf.data.pitch/10.0,(float)rf.data.yaw/10.0);
		Printftoa((float)rf.data.roll/10.0); PutChar(' ');
		Printftoa((float)rf.data.pitch/10.0); PutChar(' ');
		Printftoa((float)rf.data.yaw/10.0); PutChar(' ');

		//printf("%6.2f %6.4f\n\r",(float)rf.data.temp/100.0,(float)rf.data.press/1000.0);
		Printftoa((float)rf.data.temp/100.0); PutChar(' ');
		Printftoa((float)rf.data.press/1000.0); Puts("\n\r$");
		}
}

void RF_Send_10DOFMEMS_Data(void)
{
	volatile int i;
	
	union {
		volatile unsigned char buf[32];
		volatile struct rfmemsdata {
			unsigned long time;
			int accel[3];
			int gyro[3];
			int mag[3];
			int roll,pitch,yaw;
			int temp,press;
		} data;
	} rf;

	//isecond = (int) time.second;
	//imsec = (time.second-isecond) * 1000;
	//rf.data.time = time.hour*3600*1000 + time.minute*60*1000 + isecond*1000 + imsec;
	rf.data.time = 0;  // TBD - real time clock plus clock sync
	
	for(i=0; i<3; i++)  rf.data.accel[i] = imu.ai[i];
	for(i=0; i<3; i++)  rf.data.gyro[i] = imu.gi[i];
	for(i=0; i<3; i++)  rf.data.mag[i] = imu.mi[i];
	rf.data.roll = imu.rolli;
	rf.data.pitch = imu.pitchi;
	rf.data.yaw = imu.yawi;
	rf.data.temp = imu.tempi;
	rf.data.press = imu.pressi;
	
	if((rf_state==RF_STATE_INITIALIZED_TX)&&(rf_mode==RF_STREAM))  RF_SendData(rf.buf,PAYLOAD_SIZE);
	else if(rf_mode==RF_STREAM_ONESHOT) RF_SendDataCheck(rf.buf,PAYLOAD_SIZE);
}

void RF_Send_10DOFMEMS_Data_IMU8420(unsigned char *rfbuf)
{
	if((rf_state==RF_STATE_INITIALIZED_TX)&&(rf_mode==RF_STREAM))  RF_SendData(rfbuf,PAYLOAD_SIZE);
	else if(rf_mode==RF_STREAM_ONESHOT) {
		RF_SendDataCheck(rfbuf,PAYLOAD_SIZE);
		//Puts(" - packet sent");
		//PrintfRF(rfbuf);
		}
}

// Check for commands coming in over RF24L01 link.
// Only called if link hardware present and set to receive mode
// Any query commands turn the link around but restore it to receive
// before returning.
bool RF24L01_Process_Commands(void)
{
	volatile char outstrg[20];
	volatile int i,payloadbytes;
	volatile rf_mess=false;
	
	//printf("\n\rrf status=%x\n\r",RF_GetStatus());
	
	payloadbytes = RF_GetData();

	//if(payloadbytes>=0) printf("\n\rpayload=%d\n\r",payloadbytes);

	if(payloadbytes>0) {
		rf_mess = true;
		switch (payload[0]) {
			case 'i':
				ZeroPayload();
#ifdef SSD1306OLED
				strcpy(payload,"PicoM04L V1.0 9DOF OLED$");
#else
				strcpy(payload,"PicoM04 V1.0 9DOF$");
#endif

				RF_SendDataCheck(payload,rf_payload_size);
				break;
			// Set commands
			case 's':
				switch (payload[1]) {
					case 'i':
						//RF_SetDigital(payload);
						break;
					case 'd':
						//RF_SetDAC();
						break;
					case 't':
						//RF_SetTime();
						break;
					case 'm':
						//RF_SetMotor(payload);
						break;
					case 'r':
						if(payload[2]=='c')  {
							channel_no = payload[3];
							RF24L01_ReInit();
							//printf("rx'd sc=%d\n\r",payload[3]);
							//RD_DumpRegisters(0);
							}
						else if(payload[2]=='i') {
							device_id = payload[3];
							RF24L01_ReInit();
							//printf("rx'd si=%d  %d\n\r",payload[3],channel_no);
							//RD_DumpRegisters(0);
							}
						else if(payload[2]=='p') {
							rf_payload_size = payload[3];
							RF24L01_ReInit();
							//printf("rx'd sp=%d\n\r",payload[3]);
							//RD_DumpRegisters(0);
							}
						break;
					case 'l':
						if(payload[2]=='b') ledblink = YES;
						if(payload[2]=='o') {
							ledblink = NO; 
							GreenLedON;
							}
						if(payload[2]=='f') {
							ledblink = NO;
							GreenLedOFF;
							}
						break;
					case 's':
						//RF_SetStreamMode(payload);  // sets stream rate and sensor mix
						break;
					}
				break;
			// Get commands
			case 'g':
				switch (payload[1]) {
					case 't':
						//RF_GetTime();
						break;
					case 'a':
						//Puts("\n\rRF packet ga");
						rf_mode = RF_STREAM_ONESHOT;
						//if(memsstate!=ACTIVESTATE)  Get_10DOFMEMsData('s');
						//RF_Send_10DOFMEMS_Data();
						RF_Send_10DOFMEMS_Data_IMU8420(imu8420buf);
						rf_mode = RF_STREAM;
						break;
#ifdef FSI7013SENSOR
					case 'h':
						//strcpy(payload,"rh=");
						strcpy(payload,ftoa(si7013.humidity,2,6));
						strcat(payload," ");
						strcat(payload,ftoa(si7013.temperature,2,6));
						if(BMPstate) {
							strcat(payload," ");
							strcat(payload,ftoa(bmppressure,4,8));
							strcat(payload," ");
							strcat(payload,ftoa(bmptemperature,2,6));
							}
						strcat(payload,"$");
						rf_mode = RF_STREAM_ONESHOT;
						if((rf_state==RF_STATE_INITIALIZED_TX)&&(rf_mode==RF_STREAM))  RF_SendData(payload,PAYLOAD_SIZE);
						else if(rf_mode==RF_STREAM_ONESHOT) RF_SendDataCheck(payload,PAYLOAD_SIZE);
						rf_mode = RF_STREAM;
						break;
#endif
						case 'd':
						//RF_GetDigital(payload);
						break;
					case 'm':
						//RF_GetMEMsData();
						break;
					case 'e':
						//RF_GetEKFData();
						break;
					case 'l':
						//RF_GetLogParameters();
						break;
					}
				break;
			case 'd':
				for(i=0; i<payloadbytes; i++)  PutChar(payload[i]);
				Puts("\n\r");
			break;
		}
	}
	return rf_mess;
}
