/********************************************************************************
 *
 *   Program:  PicoM04 STM32F042       Version: 1.01
 *
 *   Author:  S. Swift				   Date: May 14, 2016
 *
 *   This test example application configures the STM32F042 as a CDC USB device
 *   using the CooCox V1.7.5 IDE and STMicroelectronics STM32F0x2_USB-FS-Device_Lib V1.0.0.
 *
 *   This software can be used without restriction as the basis for other applications
 *   and products.  The STM32F042 is a low cost ($2.50) ARM M0 processor running at 48MHz
 *   with 32K of flash, 6K SRAM, A/D, 2 SPI, 2 USARTs, I2C,  etc in 32 and 64
 *   pin QFP and QFN package types.  A good, low cost, processor.  We are building the F042
 *   into several new products using this code as a starting point.
 *
 *   Additional versions will be released with tighter integration with the Raspberry Pi
 *   before the end of June and into July.
 *
 *   Paella is capable of communicating with the Raspberry Pi USART, I2C (as master or slave)
 *   and SPI interface.  Note that the RF24L01 communicates via SPI which is attached both to
 *   STM32F042 and Raspberry Pi IO bus so either Paella controls the RF24L01 or the Raspberry
 *   Pi controls the device.  Similar with the I2C interface - either Paella controls the I2C
 *   as master with Raspberry Pi as slave or Raspberry Pi is master.  By default Paella is
 *   master of both the SPI lines and I2C lines.  STM32F042 USART2 is cross connected to the Raspberry
 *   Pi USART.
 *
 *   The application configures the USB peripheral as a CDC device, USART2 to 38,400,N,8,1,
 *   TIM3 as 1msec time delay clock, TIM15 as a usec delay timer. A simplified set of print
 *   functions are available but printf is not supported to the large flash footprint of the
 *   resulting code.  Putchar,getchar is availble for USART2.  USART2 is interrupt driven.
 *
 *   The RF24L01 code has been ported but due to the limited flash available in the F042
 *   parts not all functions will be able to work.  If you intend to use the RF24L01 features
 *   extensively you might consider using the 128K flash PaellaM07 or 256K flash PaellaM30.
 *
 *   Code to support the MPU-9150 is also provided but it's flash foot print is big.  If you
 *   intend to develop applications using the MPU-9150 you should consider using the PaellaM07
 *   or PaellaM30 versions.
 *
 *   Unless required by applicable law or agreed to in writing, software distributed under
 *   the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 *   ANY KIND, either express or implied. See the License for the specific language governing
 *   permissions and limitations under the License. Use at your own risk.
 *
 *    STM32F042K6 Descriptions:
 *       ST's STM32F042K6 operates at a maximum speed of 48MHz and features 32 Kbytes
 *       of Flash and 6 Kbytes of RAM.
 *
 *   Key Parameters:
 *       1 (16-bit) Advanced control Timers
 *       4 (16-bit) 1 (32-bit) General purpose Timers
 *       2 SPI(I2S), 1 I2C, 2 USART
 *       1 CAN, 1 USB , 1 CEC
 *       10 ext. + 3 int 12-bit synchronized ADC
 *       5-channel DMA controller
 *
 *   Copyright 2015,  SOC Robotics, Inc.
 ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2013 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
 */
/* Includes ------------------------------------------------------------------*/
#include "stm32f0xx.h"
#include "stm32f0xx_conf.h"
//#include "stm32072b_eval.h"

#include "usbd_cdc_core.h"
#include "usbd_usr.h"

#include "stdarg.h"
#include "string.h"
#include "math.h"
#include <stdio.h>
#include <stdlib.h>

#include "PicoM_Defines.h"
#include "PicoM_Externals.h"
#include "NRF24L01.h"
#include "FXOS8700.h"
#include "MMA8653.h"
#include "SI7013.h"
#include "st7735.h"
#include "colors.h"

USB_CORE_HANDLE  USB_Device_dev ;

/* Private function prototypes -----------------------------------------------*/
void LEDInit(void);
void EXTI0_Config(void);
void TIM_Config(void);
void USARTInit(unsigned long baudrate);


// Global parameters
char hexbyte[3],hexaddress[6],longaddress[9];
volatile bool packetreceived = false;

volatile unsigned int mseccountdown1,mseccountdown2,mseccountdown3;
volatile unsigned long mseccounter;
volatile char flip=0,grnflip=0,redflip=0,ledblink,updatedisplay;

// MPU-9150 9 DOF sensor parameters
volatile char mpu9150state=0;
volatile float magposition[6],mpu9150_tempoffset=MPU9150TEMPOFFSET;
extern volatile struct imudata {
	short int ai[3];
	short int gi[3];
	short int mi[3];
	short int temp;
	float magcor[3];
	float af[3];
	float gf[3];
	float mf[3];
	short int tempi;
	short int rolli;
	short int pitchi;
	short int yawi;
	short int pressi;
	float tempf;
	float roll;
	float pitch;
	float yaw;
	float press;
	char mget;
} imu;

volatile unsigned char imu8420buf[32];

// USB parameters
int usbrxheadptr=0, usbrxtailptr=0;
unsigned char usbrxbuffer[USB_RX_BUFFERSIZE];

// Usart 1 parameters
volatile int tx1headptr,tx1tailptr,rx1headptr,rx1tailptr;
volatile unsigned char rx1buffer[USART1_RX_BUFSIZE],tx1buffer[USART1_TX_BUFSIZE];

// Usart 2 parameters
int tx2headptr,tx2tailptr,rx2headptr,rx2tailptr;
unsigned char rx2buffer[USART2_RX_BUFSIZE],tx2buffer[USART2_TX_BUFSIZE];

volatile char memsstate=ACTIVESTATE;
volatile unsigned char messageBuf[20],i2ctimedout;
volatile unsigned int i2c_Timeout;

int adc_input[4];
int rf_cnt=0;


TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
TIM_OCInitTypeDef  TIM_OCInitStructure;
TIM_ICInitTypeDef  TIM_ICInitStructure;

/***********************************************
 *
 *  Initialization Code
 *
 ***********************************************/
// Initialize and start USB CDC
void USB_Init(void)
{
	USBD_Init(&USB_Device_dev,
	            &USR_desc,
	            &USBD_CDC_cb,
	            &USR_cb);
}

//  Configures LED GPIO.
void LED_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	/* Enable the GPIO_LED Clock */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	/* Configure the GPIO_LED pin */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	//  GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIO_Init(GPIOB, &GPIO_InitStructure);
}

//  Configures Port A inputs and outputs
//    PA15 - TXD2
//    PA14 - RXD2
//    PA13 - General IO
//    PA12 - D+
//    PA11 - D-
//    PA10 - General IO
//    PA9  - General IO
//    PA8  - General IO
//    PA7  - General IO
//    PA6  - General IO
//    PA5  - General IO - DAC2 (only on F072 and F303)
//    PA4  - General IO - DAC1 (only on F072 and F303)
//    PA3  - General IO - ADC3
//    PA2  - General IO - ADC2
//    PA1  - General IO - ADC1
//    PA0  - General IO - ADC0
void PortA_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	// Enable Port A outputs
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	// Configure the outputs
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15 | GPIO_Pin_10 | GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	//GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_1 ;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// Configure the GPIOA input pins
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14 | GPIO_Pin_13 | GPIO_Pin_8 | GPIO_Pin_7 | GPIO_Pin_6 |
							      GPIO_Pin_5 | GPIO_Pin_4 | GPIO_Pin_3 | GPIO_Pin_2 | GPIO_Pin_1 | GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	//GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

//  Configures Port B inputs and outputs
//    PB15 - MOSI2
//    PB14 - MISO2
//    PB13 - SCK2
//    PB12 - SS2
//    PB11 - GPIO25  - RF24L01 CSN
//    PB10 - IRQ
//    PB9  - GPIO4
//    PB8  - CE1   - RF24L01 CE
//    PB7  - SDA1
//    PB6  - SCL1
//    PB5  - General IO
//    PB4  - General IO
//    PB3  - General IO
//    PB2  - General IO
//    PB1  - General IO
//    PB0  - LED
void PortB_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	// Enable the PortB Clock
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	// Configure the outputs
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_11 | GPIO_Pin_8 |
			                      GPIO_Pin_6  | GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	//GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_1 ;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	// Configure the GPIOB input pins
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_9 | GPIO_Pin_5 | GPIO_Pin_4 |
								  GPIO_Pin_3  | GPIO_Pin_2  | GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	//GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
}

//  Configures PortC inputs and outputs
//    PC15 - OSC2
//    PC14 - OSC1
//    PC13 -
void PortC_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	// Enable the GPIO_LED Clock
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);

	// Configure PortB outputs
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15 | GPIO_Pin_14 | GPIO_Pin_13;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	//GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	//GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_1 ;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOC, &GPIO_InitStructure);

	// Configure the GPIOC input pins
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11 | GPIO_Pin_5;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	//GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOC, &GPIO_InitStructure);
}

//  Configure Port F input and outputs
//    PF1  - OSCOUT
//    PF0  - OSCIN
void PortF_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	// Enable the Port F clock
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOF, ENABLE);

	// Configure the 0 1 as outputs
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	//GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	//GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_1 ;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOF, &GPIO_InitStructure);
}

// Configure the TIM IRQ Handler.
void TIM3_Config(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	/* TIM3 clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);

	/* Enable the TIM3 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}

// Setup Timer 3 as a 1msec time delay interrupt
void TIM3_Init(void)
{
	uint16_t CCR3_Val,PrescalerValue;

	/* TIM Configuration */
	TIM3_Config();

	/* -----------------------------------------------------------------------
	TIM3 Configuration: Output Compare Timing Mode:

	In this example TIM3 input clock (TIM3CLK) is set to APB1 clock (PCLK1),
	  => TIM3CLK = PCLK1 = SystemCoreClock = 48 MHz

	To get TIM3 counter clock at 6 MHz, the prescaler is computed as follows:
	   Prescaler = (TIM3CLK / TIM3 counter clock) - 1
	   Prescaler = (PCLK1 /6 MHz) - 1

	CC3 update rate = TIM3 counter clock / CCR3_Val = 439.4 Hz
	==> Toggling frequency = 219.7 Hz

	----------------------------------------------------------------------- */

	/* Compute the prescaler value */
	PrescalerValue = (uint16_t) (SystemCoreClock  / 6000000) - 1;
	//PrescalerValue = 0;

	CCR3_Val = 1000;
	mseccountdown1 = mseccountdown2 = mseccountdown3 = 0;

	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Period = 65535;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

	/* Prescaler configuration */
	TIM_PrescalerConfig(TIM3, PrescalerValue, TIM_PSCReloadMode_Immediate);

	/* Output Compare Timing Mode configuration: Channel1 */
	//TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
	//TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

	/* Output Compare Timing Mode configuration: Channel3 */
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = CCR3_Val;

	TIM_OC3Init(TIM3, &TIM_OCInitStructure);

	TIM_OC3PreloadConfig(TIM3, TIM_OCPreload_Disable);

	/* TIM Interrupts enable */
	TIM_ITConfig(TIM3, TIM_IT_CC3, ENABLE);

	/* TIM3 enable counter */
	TIM_Cmd(TIM3, ENABLE);
}

// Set up timer 15 as usecond delay timer
void TIM15_Init(void)
{
	uint16_t CCR3_Val,PrescalerValue;

	/* TIM15 clock enable */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM15, ENABLE);

	/* Compute the prescaler value for 1usecond rate */
	PrescalerValue = (uint16_t) (SystemCoreClock  / 1000000) - 1;
	//PrescalerValue = 0;

	CCR3_Val = 0;

	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Period = 65535;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

	TIM_TimeBaseInit(TIM15, &TIM_TimeBaseStructure);

	/* Prescaler configuration */
	TIM_PrescalerConfig(TIM15, PrescalerValue, TIM_PSCReloadMode_Immediate);

	/* Output Compare Timing Mode configuration: Channel3 */
	//TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	//TIM_OCInitStructure.TIM_Pulse = CCR3_Val;

	//TIM_OC3Init(TIM15, &TIM_OCInitStructure);

	//TIM_OC1PreloadConfig(TIM15, TIM_OCPreload_Disable);

	/* TIM Interrupts enable */
	//TIM_ITConfig(TIM15, TIM_IT_CC1, ENABLE);

	/* TIM3 enable counter */
	//TIM_Cmd(TIM15, ENABLE);
}

void TIM15_Start(void)
{
	TIM_SetCounter(TIM15, 0);
	TIM_Cmd(TIM15, ENABLE);
}

void TIM15_Stop(void)
{
	TIM_Cmd(TIM15, DISABLE);
}

// Configure USART1 COM port.
void USART1_Init(unsigned long baudrate)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

    // Initialize interrupt tx rx buffers
    tx1headptr = tx1tailptr = rx1headptr = rx1tailptr = 0;

    /* USART2 configured as follow:
    - BaudRate = 115200 baud
    - Word Length = 8 Bits
    - One Stop Bit
    - No parity
    - Hardware flow control disabled (RTS and CTS signals)
    - Receive and transmit enabled
    - PA9 - TX1, PA10 - RX1   (PA14 - TX2, PA15 - RX2)
    */
    USART_InitStructure.USART_BaudRate = baudrate;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

    /* Enable GPIO clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    /* Enable USART clock */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

    /* Connect PXx to USARTx_Tx */
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_1);

    /* Connect PXx to USARTx_Rx */
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_1);

    /* Configure USART Tx, Rx as alternate function push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9 | GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /* USART configuration */
    USART_Init(USART1, &USART_InitStructure);

    USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
    USART_ITConfig(USART1, USART_IT_TXE, ENABLE);

	/* Enable the USART1 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

    /* Enable USART */
    USART_Cmd(USART1, ENABLE);
}

// Configure USART2 COM port.
void USART2_Init(unsigned long baudrate)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

    // Initialize interrupt tx rx buffers
    tx2headptr = tx2tailptr = rx2headptr = rx2tailptr = 0;

    /* USART2 configured as follow:
    - BaudRate = 115200 baud
    - Word Length = 8 Bits
    - One Stop Bit
    - No parity
    - Hardware flow control disabled (RTS and CTS signals)
    - Receive and transmit enabled
    - PA14 - TX2, PA15 - RX2     (PA9 - TX1, PA10 - RX1)
    */
    USART_InitStructure.USART_BaudRate = baudrate;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

    /* Enable GPIO clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

    /* Enable USART clock */
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

    /* Connect PXx to USARTx_Tx */
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource14, GPIO_AF_1);

    /* Connect PXx to USARTx_Rx */
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource15, GPIO_AF_1);

    /* Configure USART Tx, Rx as alternate function push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14 | GPIO_Pin_15;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /* USART configuration */
    USART_Init(USART2, &USART_InitStructure);

    USART_ITConfig(USART2, USART_IT_RXNE, ENABLE);
    USART_ITConfig(USART2, USART_IT_TXE, ENABLE);

	/* Enable the USART2 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

    /* Enable USART */
    USART_Cmd(USART2, ENABLE);
}

// I2C1 is attached to the optional OLED connector, MPU-9150 chip
// and the Raspberry Pi I2C port.  This function is enabled by default
// by PaellaM07 so the Raspberry Pi should not enable the I2C port.
// Default I2C speed is 400KHz but can operate up to 1000KHz.
void I2C1_Init(unsigned long speed)
{
	I2C_InitTypeDef  I2C1_InitStructure;
	GPIO_InitTypeDef GPIO_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure;

	I2C_DeInit(I2C1);

	// I2C Periph clock enable
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    // Configure the I2C clock source. The clock is derived from the HSI
    RCC_I2CCLKConfig(RCC_I2C1CLK_HSI);

    GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_1);
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_1);

    // Configure pins: SCL and SDA ------------------
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    // Init I2C to defaults
    I2C_StructInit(&I2C1_InitStructure);

    I2C1_InitStructure.I2C_AnalogFilter = I2C_AnalogFilter_Enable;
    I2C1_InitStructure.I2C_DigitalFilter = 0x00;
    I2C1_InitStructure.I2C_OwnAddress1 = 0x00;
    I2C1_InitStructure.I2C_Mode = I2C_Mode_I2C;
    I2C1_InitStructure.I2C_Ack = I2C_Ack_Enable;

    /* I2C TIMING Register define when I2C clock source is SYSCLK */
    /* I2C TIMING is calculated in case of the I2C Clock source is the SYSCLK = 72 MHz */
    /* When using SFOX8700 set TIMING to 0x00C4092A to reach 1 MHz speed (Rise time = 26ns, Fall time = 2ns) */
    /* When using slow devices set TIMING to 0xC062121F to reach 100 KHz speed (Rise time = 640ns, Fall time = 20ns) */

    //I2C_InitStructure.I2C_Timing = 0x0x00C4092A;  // 1MHz
    //I2C1_InitStructure.I2C_Timing = 0xC062121F;  // 100KHz
    //I2C1_InitStructure.I2C_Timing = 0x20310A0D;  // 400KHz
    //I2C1_InitStructure.I2C_Timing = 0x50330309; // 400KHz - other source
    I2C1_InitStructure.I2C_Timing = speed;
    I2C1_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;

    I2C_Init(I2C1, &I2C1_InitStructure);

	// Configure the I2C interrupt priority
	//NVIC_InitStructure.NVIC_IRQChannel = I2C1_EV_IRQn;
	//NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	//NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	//NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	//NVIC_Init(&NVIC_InitStructure);

	// Enable I2C
	I2C_Cmd(I2C1, ENABLE);

	// Enable Interrupts
	//I2C_ITConfig(I2C1, I2C_IT_ADDRI | I2C_IT_RXI, ENABLE);
}

// SPI1 is available on PA5,6,7 or PB3,4,5
// Th3 function defaults to PB3,4,5 - use PB2 as CS but is
// called by default by the PaellaM07_Init function
void SPI1_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    SPI_InitTypeDef SPI_InitStructure;

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB | RCC_AHBPeriph_GPIOE, ENABLE);

    // Configure IO for PB3,4,5
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource3, GPIO_AF_0); // SPI SCK
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource4, GPIO_AF_0); // SPI MISO
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource5, GPIO_AF_0); // SPI MOSI

    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;//GPIO_PuPd_DOWN;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    SPI_I2S_DeInit(SPI1);
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8;
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
    SPI_Init(SPI1, &SPI_InitStructure);

    SPI_RxFIFOThresholdConfig(SPI1, SPI_RxFIFOThreshold_QF);

    SPI_Cmd(SPI1, ENABLE);

    // Ensure RXNE is initially clear
    SPI_ReceiveData8(SPI1);
}

// SPI2 is on PortB and is attached to the RF24L01 port pins and Raspberry Pi
// expansion connector.  Control of the RF24L01 is shared by lines on the
// Raspberry Pi and Paella - only the STM32F042 or Raspberry Pi should
// communicate with the RF24L01 - not both at the same time.
//   PB13 - SCK2
//   PB14 - MISO2
//   PB15 - MOSI2
//   RF24L01 CE - PB11
//   RF24L01 CSN - PB12
void SPI2_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    SPI_InitTypeDef SPI_InitStructure;

    if(1) {
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    // Configure IO for nRF24L01 SPI SCK, MISO, MOSI
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource13, GPIO_AF_0); // SPI SCK
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource14, GPIO_AF_0); // SPI MISO
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource15, GPIO_AF_0 ); // SPI MOSI

    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;//GPIO_PuPd_DOWN;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_3;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

	// GPIO SPI2 CS - CSN-PB12, CE-PB11
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_11;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	// OType, Speed and PuPd are the same
	GPIO_Init(GPIOB, &GPIO_InitStructure);

    SPI_I2S_DeInit(SPI2);
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
    //SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8; // 4mbit rate
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_16;  // 2mbit rate
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
    SPI_Init(SPI2, &SPI_InitStructure);

    SPI_RxFIFOThresholdConfig(SPI2, SPI_RxFIFOThreshold_QF);

    //printf("\n\rSPI_BaudRatePrescaler_8=%d\n\r",SPI_BaudRatePrescaler_8);

    SPI_Cmd(SPI2, ENABLE);

    // Ensure RXNE is initially clear
    SPI_ReceiveData8(SPI2);
	}
	// Bit bang SPI function
	else {
	    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

		// GPIO SPI2 CS - CE-PB11, CSN-PB12, SCK2-PB13, MOSI2-PB15
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_15;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
		// OType, Speed and PuPd are the same
		GPIO_Init(GPIOB, &GPIO_InitStructure);

		// GPIO SPI2 Input - MISO2-PB14
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
		// OType, Speed and PuPd are the same
		GPIO_Init(GPIOB, &GPIO_InitStructure);

		MOSI2HIGH;
		SCK2LOW;
		}
}

uint8_t SPI1_transfer_byte(uint8_t byte)
{
	while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET);
	// Send a Byte through the SPI peripheral
	SPI_SendData8(SPI1, byte);
	while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET);
	return (uint8_t)SPI_ReceiveData8(SPI1);
}

// Bit bang version - pretty slow
uint8_t SPI2_transfer_byte_old(uint8_t outbyte)
{
	unsigned char inbyte,mask=0x80;
	int i;

	inbyte = 0;
	for(i=0; i<8; i++) {
		if(outbyte&mask) MOSI2HIGH;
		else MOSI2LOW;
		delay_us(1);
		SCK2HIGH;
		if(GPIO_ReadInputDataBit(GPIOB,GPIO_Pin_14)) inbyte |= mask;
		delay_us(1);
		SCK2LOW;
		//delay_us(2);
		mask = mask >> 1;
		}
	MOSI2HIGH;
	return inbyte;
}

uint8_t SPI2_transfer_byte(uint8_t byte)
{
	unsigned int flgstatus;

	//Puts("in rf transfer byte = ");
	//PrintInt(byte);
	//Puts("\n\r");
	//delay_ms(20);

	while((flgstatus=SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_TXE)) == RESET) {
		//Puts("\rFirst Flag status = ");
		//PrintInt(flgstatus);
		}
	//delay_ms(20);
	//Puts("\n\rFirst Flag status = ");
	//PrintInt(flgstatus);
    //Puts("\n\r");
	//delay_ms(20);

	// Send a Byte through the SPI peripheral
	SPI_SendData8(SPI2, byte);
	while((flgstatus=SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_RXNE)) == RESET){
		//Puts("\rSecond Flag status = ");
		//PrintInt(flgstatus);
		//printf("\rFlag status - second call=%x ",flgstatus);
		}
	//delay_ms(20);

	//Puts("\n\rFirst Flag status = ");
	//PrintInt(flgstatus);
	//delay_ms(20);

	//Puts("\n\rReturning from transfer byte\n\r");

	return (uint8_t)SPI_ReceiveData8(SPI2);
}

// Assume PA0-3 are ADC inputs
void Init_ADC()
{
	int calibration_value;

	ADC_InitTypeDef ADC_init_structure; //Structure for ADC configuration
	GPIO_InitTypeDef GPIO_initStructre; //Structure for analog input pin
	//ADC_CommonInitTypeDef  ADC_CommonInitStructure;

	//RCC_ADCCLKConfig(RCC_ADC12PLLCLK_Div2);

	//Clock configuration
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1,ENABLE); //The ADC12 is connected the APB peripheral bus thus we will use its clock source
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA,ENABLE); //Clock for the ADC port!! Do not forget about this one ;)

	//Analog pin configuration
	GPIO_initStructre.GPIO_Pin = GPIO_Pin_3|GPIO_Pin_2|GPIO_Pin_1|GPIO_Pin_0;
	GPIO_initStructre.GPIO_Mode = GPIO_Mode_AN; //The PA0-3 pin is configured in analog mode
	GPIO_initStructre.GPIO_PuPd = GPIO_PuPd_NOPULL; //We don't need any pull up or pull down
	GPIO_Init(GPIOA,&GPIO_initStructre); //Affecting the port with the initialization structure configuration

	//ADC_DeInit(ADC1);
	ADC_StructInit(&ADC_init_structure);

	//ADC_VoltageRegulatorCmd(ADC1, ENABLE);
	delay_us(10);

	//ADC_SelectCalibrationMode(ADC1, ADC_CalibrationMode_Single);
	//ADC_StartCalibration(ADC1);

	//while(ADC_GetCalibrationStatus(ADC1) != RESET );
	//calibration_value = ADC_GetCalibrationValue(ADC1);


	/* ADC configuration */
	//ADC_CommonInitStructure.ADC_Mode = ADC_Mode_Independent;
	//ADC_CommonInitStructure.ADC_Clock = ADC_Clock_AsynClkMode;
	//ADC_CommonInitStructure.ADC_DMAAccessMode = ADC_DMAAccessMode_Disabled; // Not Multi Mode
	//ADC_CommonInitStructure.ADC_DMAMode = ADC_DMAMode_Circular;
	//ADC_CommonInitStructure.ADC_TwoSamplingDelay = 0;
	//ADC_CommonInit(ADC1,&ADC_CommonInitStructure);

	//ADC structure configuration
	ADC_init_structure.ADC_DataAlign = ADC_DataAlign_Right; //data converted will be shifted to right
	ADC_init_structure.ADC_Resolution = ADC_Resolution_12b; //Input voltage is converted into a 12bit number giving a maximum value of 4096
	ADC_init_structure.ADC_ContinuousConvMode = DISABLE; //the conversion is continuous, the input data is converted more than once
	ADC_init_structure.ADC_ExternalTrigConv = ADC_ExternalTrigConvEdge_None; // conversion is synchronous with TIM1 and CC1 (actually I'm not sure about this one :/)
	ADC_init_structure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None; //no trigger for conversion
	ADC_init_structure.ADC_ScanDirection = ADC_ScanDirection_Upward; //The scan is configured
	ADC_Init(ADC1,&ADC_init_structure); //Initialize ADC with the previous configuration


	//Enable ADC conversion
	ADC_Cmd(ADC1,ENABLE);
}

// Get ADC data
int ADC_Convert(unsigned char channel)
{
	//channel = ADC_Channel_10;
	//Select the channel to be read from
	ADC_RegularChannelConfig(ADC1,channel,1,ADC_SampleTime_1_5Cycles);
	//ADC_RegularChannelConfig(ADC1,channel,1,ADC_SampleTime_144Cycles);

	//Start the conversion
	ADC_StartConversion(ADC1);
	//printf("Stat=%x ",ADC1->ISR);
	//delay_us(300);

	//printf("ADC_Status=%x ",ADC1->ISR);

	while(!ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC)); //Processing the conversion

	return ADC_GetConversionValue(ADC1); //Return the converted data
}

void Sample_ADC_Channels(void)
{
	adc_input[0] = ADC_Convert(ADC_Channel_1);
	adc_input[1] = ADC_Convert(ADC_Channel_2);
	adc_input[2] = ADC_Convert(ADC_Channel_3);
	adc_input[3] = ADC_Convert(ADC_Channel_4);
}

/***********************************************
 *  ISRs
 ***********************************************/
void TIM3_IRQHandler(void)
{
    if (TIM_GetITStatus(TIM3, TIM_IT_CC3) != RESET)  {
		TIM_ClearITPendingBit(TIM3, TIM_IT_CC3);

		if(mseccountdown1) mseccountdown1--;
		if(mseccountdown2) mseccountdown2--;
		if(mseccountdown3) mseccountdown3--;
		TIM_SetCounter(TIM3, 0);
    	}
}

void USART1_IRQHandler(void)
{
	// Check receive interrupt
    if (USART_GetITStatus(USART1, USART_IT_RXNE) != RESET)  {
    	USART_ClearITPendingBit(USART1, USART_IT_RXNE);
    	//LED1LOW;
		rx1buffer[rx1headptr++] = (uint8_t)(USART1->RDR & (uint16_t)0x00FF);
		if(rx1headptr>=USART1_RX_BUFSIZE) rx1headptr = 0;
		if(rx1headptr==rx1tailptr) {
			rx1tailptr++;
			if(rx1tailptr<=USART1_RX_BUFSIZE) rx1tailptr = 0;
			}
    	}

    // Check transmit interrupt
    if (USART_GetITStatus(USART1, USART_IT_TXE) != RESET)  {
    	USART_ClearITPendingBit(USART1, USART_IT_TXE);
    	//LED2LOW;
		USART1->TDR = tx1buffer[tx1tailptr++];
		if(tx1tailptr==USART1_TX_BUFSIZE) tx1tailptr = 0;
		// Turn off interrupt as no more data to transmit
		if(tx1tailptr==tx1headptr)  USART_ITConfig(USART1, USART_IT_TXE, DISABLE);
    	}
}

/*
void USART2_IRQHandler(void)
{
	// Check receive interrupt
    if (USART_GetITStatus(USART2, USART_IT_RXNE) != RESET)  {
    	USART_ClearITPendingBit(USART2, USART_IT_RXNE);
    	//LED1LOW;
		rx2buffer[rx2headptr++] = (uint8_t)(USART2->RDR & (uint16_t)0x00FF);
		if(rx2headptr==USART2_RX_BUFSIZE) rx2headptr = 0;
		if(rx2headptr==rx2tailptr) {
			rx2tailptr++;
			if(rx2tailptr==USART2_RX_BUFSIZE) rx2tailptr = 0;
			}
    	}
    // Check transmit interrupt
    if (USART_GetITStatus(USART2, USART_IT_TXE) != RESET)  {
    	USART_ClearITPendingBit(USART2, USART_IT_TXE);
    	//LED2LOW;
		USART2->TDR = tx2buffer[tx2tailptr++];
		if(tx2tailptr==USART2_TX_BUFSIZE) tx2tailptr = 0;
		// Turn off interrupt is not more data to transmit
		if(tx2tailptr==tx2headptr)  USART_ITConfig(USART2, USART_IT_TXE, DISABLE);
    	}
}
*/
/***********************************************
 *  I2C Functions
 ***********************************************/
#define INIT_TIMEOUT i2c_Timeout = I2C_LONG_TIMEOUT
#define CHECK_TIMEOUT if((i2c_Timeout--) == 0) return I2C_TIMEOUT_UserCallback()

uint8_t I2C_TIMEOUT_UserCallback(void)
{
	i2ctimedout = YES;
	return i2ctimedout;
}

//  Write to the specified number of bytes to the I2C device
uint8_t I2C_WriteBytes(I2C_TypeDef* I2Cx,uint8_t twiaddress,uint8_t nbytes)
{
	int i=0;
	i2ctimedout = NO;

	// Test on BUSY Flag
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_BUSY) != RESET) {CHECK_TIMEOUT;}

	// Configure slave address, nbytes, reload, end mode and start or stop generation
	I2C_TransferHandling(I2Cx, twiaddress<<1, nbytes, I2C_AutoEnd_Mode, I2C_Generate_Start_Write);

	while(nbytes--) {
		// Wait until TXIS flag is set
		INIT_TIMEOUT;
		while(I2C_GetFlagStatus(I2Cx, I2C_ISR_TXIS) == RESET)  {CHECK_TIMEOUT;}

		// Send Register address
		I2C_SendData(I2Cx, (uint8_t)messageBuf[i++]);
		}

	// Wait until STOPF flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_STOPF) == RESET) {CHECK_TIMEOUT;}

	// Clear STOPF flag
	I2C_ClearFlag(I2Cx, I2C_ICR_STOPCF);

	return I2C_OK;
}

// Read the specified number of bytes from the I2C device.
// I2Cx - Select I2C1 or I2C2
// twiaddress - I2C address (must shift left one bit
// nbytes - Number of bytes to write
uint8_t I2C_ReadBytes(I2C_TypeDef* I2Cx,uint8_t twiaddress,uint8_t nbytes)
{
	int i=0;
	i2ctimedout = NO;

	// Test on BUSY Flag
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_BUSY) != RESET) {CHECK_TIMEOUT;}

	// Configure slave address, nbytes, reload, end mode and start or stop generation
	I2C_TransferHandling(I2Cx, twiaddress<<1, nbytes, I2C_AutoEnd_Mode, I2C_Generate_Start_Read);

	while(nbytes--) {
		// Wait until RXNE flag is set
		INIT_TIMEOUT;
		while(I2C_GetFlagStatus(I2Cx, I2C_ISR_RXNE) == RESET) {CHECK_TIMEOUT;}

		// Read data from RXDR
		messageBuf[i++]= I2C_ReceiveData(I2Cx);
		}

	// Wait until STOPF flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_STOPF) == RESET) {CHECK_TIMEOUT;}

	// Clear STOPF flag
	I2C_ClearFlag(I2Cx, I2C_ICR_STOPCF);

	// return a Reg value
	return messageBuf[0];
}

// Read the specified register from the I2C device using a repeated start - combined
// I2Cx - Select I2C1 or I2C2
// twiaddress - I2C address (must shift left one bit
// RegId - specifies the device register to be read.
// nbytes - number of bytes to read
uint8_t I2C_WriteReadBytes(I2C_TypeDef* I2Cx,uint8_t twiaddress,uint8_t RegId,uint8_t nbytes)
{
	int i=0;
	i2ctimedout = NO;

	// Test on BUSY Flag
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_BUSY) != RESET) {CHECK_TIMEOUT;}

	// Configure slave address, nbytes, reload, end mode and start or stop generation
	I2C_TransferHandling(I2Cx, twiaddress<<1, 1, I2C_SoftEnd_Mode, I2C_Generate_Start_Write);

	// Wait until TXIS flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_TXIS) == RESET) {CHECK_TIMEOUT;}

	// Send Register address
	I2C_SendData(I2Cx, (uint8_t)RegId);

	// NEED TO GENERATE RESTART CONDITION HERE
	//I2C_GenerateSTART(I2C1, ENABLE);
	//while(I2C_GetFlagStatus(I2C1,I2C_FLAG_BUSY));

	// Wait until TC flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_TC) == RESET) {CHECK_TIMEOUT;}

	// Configure slave address, nbytes, reload, end mode and start or stop generation
	I2C_TransferHandling(I2Cx, twiaddress<<1, nbytes, I2C_AutoEnd_Mode, I2C_Generate_Start_Read);
	//I2C_GenerateSTART(I2C1, ENABLE);

	while(nbytes--) {
		// Wait until RXNE flag is set
		INIT_TIMEOUT;
		while(I2C_GetFlagStatus(I2Cx, I2C_ISR_RXNE) == RESET) {CHECK_TIMEOUT;}

		// Read data from RXDR
		messageBuf[i++]= I2C_ReceiveData(I2Cx);
		}

	// Wait until STOPF flag is set
	INIT_TIMEOUT;
	while(I2C_GetFlagStatus(I2Cx, I2C_ISR_STOPF) == RESET) {CHECK_TIMEOUT;}

	// Clear STOPF flag
	I2C_ClearFlag(I2Cx, I2C_ICR_STOPCF);

	// return a Reg value
	return messageBuf[0];
}

/***********************************************
 *  Utility Functions
 ***********************************************/
// Not very accurate - replace with timer based delay
//  TBD - Get TIM15 to work - not sure why it doesn't work
//        on STMF042 while it does on F072
void delay_us(unsigned int ult)
{
	volatile unsigned int i,j;

	for(i=0; i<ult; ult--) {
		j = i;
		}
/*

  TIM15_Start();
	  Puts("\n\rStart TIM15\n\r");
	  while(TIM_GetCounter(TIM15) < ult)  {
		  PrintInt(ult);
	      }
	  TIM15_Stop();
 */
}


void delay_ms(unsigned int ult)
{
	mseccountdown1 = ult;
	while(mseccountdown1) ;
}

void Toggle_Leds(int grncnt)
{
	if(!mseccountdown2) {
		mseccountdown2 = grncnt;
		if(grnflip) {
			LED1LOW;
			grnflip = 0;
			}
		else {
			LED1HIGH;
			grnflip = 1;
			}
		}
}

/*************************************************************************
 *   USB character IO functions
 *************************************************************************/
int GetCharnw1(void)
{
	unsigned char inchar;
	int intchar;

	if(rx1headptr!=rx1tailptr) {
		inchar = rx1buffer[rx1tailptr++];
		intchar = inchar;
		if(rx1tailptr>=USART1_RX_BUFSIZE) rx1tailptr = 0;
		return intchar;
		}
	else return -1;
}

unsigned char GetChar1(void)
{
	int intchar;

	while((intchar=GetCharnw1())==-1) ;
	return (unsigned char) intchar;
}

void PutChar1(unsigned char outchar)
{
	if(tx1headptr==tx1tailptr) {
		tx1buffer[tx1headptr++] = outchar;
		if(tx1headptr>=USART1_TX_BUFSIZE) tx1headptr = 0;
		USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
		}
	else {
		tx1buffer[tx1headptr++] = outchar;
		if(tx1headptr>=USART1_TX_BUFSIZE) tx1headptr = 0;
		if(tx1tailptr==tx1headptr) {
			tx1tailptr++;
			if(tx1tailptr>=USART1_TX_BUFSIZE) tx1tailptr = 0;
			}
		}
}

void Puts1(unsigned char *outstrg)
{
	int i;

	for(i=0; i<strlen(outstrg); i++) PutChar1(outstrg[i]);
}

int GetCharnw2(void)
{
	int intchar;

	if(rx2headptr!=rx2tailptr) {
		intchar = rx2buffer[rx2tailptr++];
		if(rx2tailptr==USART2_RX_BUFSIZE) rx2tailptr = 0;
		return intchar;
		}
	else return -1;
}

unsigned char GetChar2(void)
{
	int intchar;

	while((intchar=GetCharnw2())==-1) ;
	return (unsigned char) intchar;
}

void PutChar2(unsigned char outchar)
{
	if(tx2headptr==tx2tailptr) {
		tx2buffer[tx2headptr++] = outchar;
		if(tx2headptr==USART2_TX_BUFSIZE) tx2headptr = 0;
		USART_ITConfig(USART2, USART_IT_TXE, ENABLE);
		}
	else {
		tx2buffer[tx2headptr++] = outchar;
		if(tx2headptr==USART2_TX_BUFSIZE) tx2headptr = 0;
		if(tx2tailptr==tx2headptr) {
			tx2tailptr++;
			if(tx2tailptr==USART2_TX_BUFSIZE) tx2tailptr = 0;
			}
		}
}

void Puts2(unsigned char *outstrg)
{
	int i;

	for(i=0; i<strlen(outstrg); i++) PutChar2(outstrg[i]);
}

/*************************************************************************
 *   USB character IO functions
 *************************************************************************/
extern uint8_t  APP_Rx_Buffer []; /* Write CDC received data in this buffer.
                                     These data will be sent over USB IN endpoint
                                     in the CDC core functions. */
extern uint32_t APP_Rx_ptr_in;    /* Increment this pointer or roll it back to
                                     start address when writing received data
                                     in the buffer APP_Rx_Buffer. */

/**
  * @brief  VCP_DataTx
  *         CDC received data to be send over USB IN endpoint are managed in
  *         this function.
  * @param  Buf: Buffer of data to be sent
  * @param  Len: Number of data to be sent (in bytes)
  * @retval Result of the operation: USBD_OK if all operations are OK else VCP_FAIL
  */
static uint16_t VCP_DataTx2(uint8_t* Buf, uint32_t Len)
{
	int i;

    for(i=0; i<Len; i++) {
    	APP_Rx_Buffer[APP_Rx_ptr_in++] = Buf[i];
    	//PutChar(Buf[i]);
    	if(APP_Rx_ptr_in == APP_RX_DATA_SIZE)  APP_Rx_ptr_in = 0;
    	}
  return USBD_OK;
}

// USB putchar function
void PutChar(unsigned char outchar)
{
	unsigned char outbuf[2];

	outbuf[0] = outchar;
	VCP_DataTx2(outbuf,1);
}

// USB get char no wait
int GetCharnw(void)
{
	unsigned char inchar;
	int intchar;

	if(usbrxheadptr!=usbrxtailptr) {
		inchar = usbrxbuffer[usbrxtailptr++];
		intchar = inchar;
		if(usbrxtailptr>=USB_RX_BUFFERSIZE) usbrxtailptr = 0;
		return intchar;
		}
	else return -1;
}

// USB get char wait
unsigned char GetChar(void)
{
	int intchar;

	while((intchar=GetCharnw())==-1) ;
	return (unsigned int) intchar;
}

void Puts(unsigned char *outstrg)
{
	int i;

	for(i=0; i<strlen(outstrg); i++) PutChar(outstrg[i]);
}


 // Formatted output - memory expensive - will blow memory limit if used
int printf(const char *fmt, ...)
{
	va_list args;
	int i=0;
	/*	char printbuffer[150];

	va_start (args, fmt);
	i = vsnprintf (printbuffer, 150, fmt, args);
	va_end (args);

	// Print the string
	VCP_DataTx2(printbuffer,i);
*/
	return i;
}

/*************************************************************************
 *
 *   Low memory usage print int and float
 *    *
 *************************************************************************/
// Convert ascii float to float binary
double atofloat(char *s)
{
	double a = 0.0;
	int e = 0;
	int c;
	while ((c = *s++) != '\0' && isdigit(c)) {
		a = a*10.0 + (c - '0');
	}
	if (c == '.') {
		while ((c = *s++) != '\0' && isdigit(c)) {
			a = a*10.0 + (c - '0');
			e = e-1;
		}
	}
	if (c == 'e' || c == 'E') {
		int sign = 1;
		int i = 0;
		c = *s++;
		if (c == '+')
			c = *s++;
		else if (c == '-') {
			c = *s++;
			sign = -1;
		}
		while (isdigit(c)) {
			i = i*10 + (c - '0');
			c = *s++;
		}
		e += i*sign;
	}
	while (e > 0) {
		a *= 10.0;
		e--;
	}
	while (e < 0) {
		a *= 0.1;
		e++;
	}
	return a;
}

// Reverse string s in place
void reverse(char s[])
{
    int i, j;
    char c;

    for (i = 0, j = strlen(s)-1; i<j; i++, j--) {
        c = s[i];
        s[i] = s[j];
        s[j] = c;
    }
}

//  Convert byte to ASCII hex
void BytetoHex(unsigned char byte)
{
    unsigned char inbyte;

	inbyte = byte & 0x0f;
	if(inbyte > 9) hexbyte[1] = (inbyte - 9) + 0x40;
	else hexbyte[1] = 0x30 + inbyte;
	inbyte = byte & 0xf0;
	inbyte >>= 4;
	if(inbyte >9) hexbyte[0] = (inbyte - 9) + 0x40;
	else hexbyte[0] = 0x30 + inbyte;
    hexbyte[2] = 0x00;
}

// Convert word to ASCII Hex
void AddresstoHex(unsigned int address)
{
 	 unsigned char lowbyte,highbyte;

	 lowbyte = address & 0x00FF;
	 BytetoHex(lowbyte);
	 hexaddress[2] = hexbyte[0];
	 hexaddress[3] = hexbyte[1];
	 highbyte = address/256;
	 BytetoHex(highbyte);
	 hexaddress[0] = hexbyte[0];
	 hexaddress[1] = hexbyte[1];
	 hexaddress[4] = 0x00;
}

// Convert long to ASCII Hex
void LongtoHex(unsigned long address)
{
 	 unsigned char lowbyte,byte2,byte3,highbyte;

	 lowbyte = address & 0x000000FF;
	 BytetoHex(lowbyte);
	 longaddress[6] = hexbyte[0];
	 longaddress[7] = hexbyte[1];
	 address >>= 8;
	 byte2 = address & 0x000000FF;
	 BytetoHex(byte2);
	 longaddress[4] = hexbyte[0];
	 longaddress[5] = hexbyte[1];
	 address >>= 8;
	 lowbyte = address & 0x000000FF;
	 BytetoHex(byte3);
	 longaddress[2] = hexbyte[0];
	 longaddress[3] = hexbyte[1];
	 address >>= 8;
	 highbyte = address & 0x000000FF;
	 BytetoHex(highbyte);
	 longaddress[0] = hexbyte[0];
	 longaddress[1] = hexbyte[1];
	 longaddress[8] = 0x00;
}

void PrintHexByte(unsigned int address)
{
	char outstrg[5];

	AddresstoHex(address);
	hexaddress[0] = hexaddress[2];
	hexaddress[1] = hexaddress[3];
	hexaddress[2] = 0;
	if(address==0) {
		outstrg[0] = '0';
		outstrg[1] = 0x00;
		Puts(outstrg);
		}
	else Puts(hexaddress);
}

void PrintHexWord(unsigned int address)
{
	char outstrg[5];

	AddresstoHex(address);
	if(address==0) {
		outstrg[0] = '0';
		outstrg[1] = 0x00;
		Puts(outstrg);
		}
	else Puts(hexaddress);
}

void PrintHexLong(unsigned long address)
{
	char outstrg[5];

	LongtoHex(address);
	if(address==0) {
		outstrg[0] = '0';
		outstrg[1] = 0x00;
		Puts(outstrg);
		}
	else Puts(longaddress);
}

// Low memory print int function
void PrintInt(int i)
{
	char outstrg[10];

	if(i==0) {
		outstrg[0] = '0';
		outstrg[1] = 0x00;
		}
	else itoa(i,outstrg,10);
	Puts(outstrg);
}

// Low memory print float function
void Printftoa(float f)
{
	char outstrg[20];

	ftoa(f,4,8,outstrg);
	Puts(outstrg);
}

void Init_RF24L01(void)
{
	//rf_mode = RF_STATE_NOTINITIALIZED;
	rf_device_class = DEVICE_CLASS;
	device_id = PICOM_DEVICE_ID;
	rf_payload_size = PAYLOAD_SIZE;
	channel_no = CHANNEL_NO;
	rf24L01_Init(RFRECEIVER);
 	//rf24L01_Init(RFSENDER);
	if(rf_state==RF_STATE_INITIALIZED_TX)  rf_mode = RF_STREAM;  // Enable RF stream mode by default
}

// Initialize the processors peripherals
void STMF042_Init(long baudrate)
{
	// Init Ports
	LED_Init();   // Leds on PB0
	PortA_Init();
	PortB_Init();
	PortC_Init();
	PortF_Init();

	TIM3_Init();  // 1msec pulse interrupt
	TIM15_Init(); // usec delay timer

	// Initialize I2C at Fast Speed to ensure correct handling of default I2C devices
	//I2C1_Init(I2C100KHZ);
	//I2C2_Init(I2C100KHZ);
	I2C1_Init(I2C400KHZ);
	//I2C2_Init(I2C400KHZ);

	SPI2_Init();

	// Turn LEDs off
	LED1OFF;

	// Setup USART
	USART1_Init(baudrate);
	//USART2_Init(baudrate);

	// Start the USB CDC Subsystem
	USB_Init();

	// See what I2C devices there are
	Enumerate_TWI_Devices(QUIET);

	// Setup MPU9150 if enabled
	//if(mpu9150state) MPU9150_Init();

	// Check for RF24L01 and initialize if present
	delay_ms(12);  // Let RF24L01 come out of power on reset

	Init_RF24L01();

#ifdef SSD1306OLED
	delay_ms(40);
	ssd1306_Init();
	ssd1306_clear(0);
	#ifdef DISPLAYSENSORDATA
		updatedisplay = ACTIVESTATE;
		updaterate = DISPLAYPDATERATE;
	#else
		updatedisplay = INACTIVE;
	#endif
#endif

#ifdef DISPLAYSENSORDATA
	memsstate = ACTIVESTATE;
#else
	memsstate = INACTIVE;  // Enable once RF24L01 is interrupt driven
#endif
}


void Signon(void)
{
	Puts("\n\rPicoM04 9DOF Wireless Sensor Processor V1.00\n\r");
#ifdef SSD1306OLED
	ssd1306_clear(0);
	ssd1306_printf(0,0,"PicoM04 V1.0");
#endif

	Puts("Copyright 2016 SOC Robotics, Inc.\n\r");
	Enumerate_TWI_Devices(VERBOSE);
	Puts("\n\r");
	Display_RF_Status();
	Puts("\n\r>");
}

void Check_RF_Connection(void)
{
	if(rf_state==RF_STATE_INITIALIZED_RX)  {
		// true if message came in
		if(RF24L01_Process_Commands()) {
			rf_inmessage = true;
			rf_cnt = mseccounter;
			}
		}

	// Check if inbound rf message occurred
	if(rf_inmessage) {
		if(mseccounter>rf_cnt+1500) {
			rf_inmessage = false;
			}
		}
}

void Flush_Uart1(void)
{
	while(GetCharnw1()!=-1) ;
}

// Check is any data packets on Uart1
void Process_Uart1(void)
{
	char inchar,inchar2;
	int i,intchar;

	if((intchar=GetCharnw1())!=-1) {
		inchar = (unsigned char) intchar;
		switch(inchar) {
			case ESC:
				inchar2 = GetChar1();
				if(inchar2=='i') {
					Puts1("PicoM V1.00$");
					//Puts("PicoM V1.00");
					}
				break;
			case 's':
				inchar2 = GetChar1();
				switch (inchar2) {
					case 'i':
						for(i=0; i<32; i++) imu8420buf[i] = GetChar1();
						packetreceived = true;
						break;
					case 'g':
						// Receive gps data
						//for(i=0; i<32; i++) imu8420buf[i] = GetChar1();
						//packetreceived = true;
						break;
					}
				break;
			case 'g':
				break;
			}
		// Clear out extra characters just in case
		Flush_Uart1();
		}
}

void Dump_Usart_Registers(void)
{
	Puts("\n\rUsart Status Registers");
	Puts("\n\rUSART1->CR1="); PrintHexLong(USART1->CR1);
	Puts("\n\rUSART1->CR2="); PrintHexLong(USART1->CR2);
	Puts("\n\rUSART1->CR3="); PrintHexLong(USART1->CR3);
	Puts("\n\r$");
}

int main(void)
{
	char inchar,inchar2,outstrg[20];
	unsigned char rfbuf[32];
	int i,intchar,status,intvalue,loopcnt=0;
	float a;
	bool printimu=false;

	// Initialize various peripherals including interrupt based USART1
	STMF042_Init(57200);
	IMU8420_Init();

	delay_ms(50);
	Signon();

	// Main processing loop - handle serial data on USB line
    while(1) {

		// Check for commands coming in on the RF24L01 link and process them
		// TBD - convert to interrupt function
		Check_RF_Connection();

		// Update MEMs sensor data - keep tracking slow update sensors like BMP180 barometer
		//if(memsstate==ACTIVESTATE) MEMS_Sensor_Update();

		// Check if any data packets need processing on Uart1
		// This function supports data packets coming from IMU8420 on Uart1
		Process_Uart1();
		if(packetreceived) {
			if(printimu) {
				PrintfRF(imu8420buf);
				}
			//else {
			//	PrintInt(loopcnt);
			//	PutChar(' ');
			//	PrintInt(loopcnt++); PutChar(' ');
			//	}
			packetreceived = false;
			}

		// Update display if present -
		//if(updatedisplay==ACTIVESTATE) Update_Display();

    	if((intchar=GetCharnw())!=-1) inchar = (unsigned char) intchar;
    	else inchar = 0;

    	if(inchar!=0) PutChar(inchar);

    	switch (inchar) {
			case ESC:
				Puts("ESC");
				PutChar(inchar=GetChar());
				GetChar();
				if(inchar=='i') Puts("\n\rPicoM04  V1.00\n\r$");
				else Puts("\n\r$");
				break;
			case 's':
				Signon();
				break;
			case 'r':
				Puts("\n\rRe-Initialize RF24L01");
				Init_RF24L01();
				Puts("\n\r$");
				break;
			case 'e':
				Enumerate_TWI_Devices(VERBOSE);
				Puts("\n\r$");
				break;
			case '1':
				IMU8420_Init();
				PrintfRF(imu8420buf);
				break;
			case '2':
				a = atofloat("6.34");
				Puts("\n\rf=");
				Printftoa(a); PutChar(' '); Printftoa(0.34);
				Puts("\n\r$");
				break;
			case '3':
				Puts("\n\rDelay 10 seconds\n\r");
				for(i=0; i<10000; i++) delay_us(1000);
				Puts("\n\r$");
				break;
			case '4':
				Puts("\n\rLoop back Uart1\n\r");
				inchar2 = ' ';
				bool dontprint = false;
				while(inchar2!='e') {
					intchar = GetCharnw();
					if(intchar=='e') break;
					if(intchar=='i') dontprint = true;
					//delay_ms(20);
					//PrintInt(intchar);
					if(intchar!=-1) {
						inchar2 = (unsigned char) intchar;
						PutChar1(inchar2);
						//delay_ms(20);
						if(!dontprint)  Dump_Usart_Registers();
						}
					intchar = GetCharnw1();
					if(intchar!=-1) PutChar((unsigned char) intchar);
					}
				Puts("\n\r$");
				break;
			case '5':
				Dump_Usart_Registers();
				break;
			case '6':
				if(printimu) printimu = false;
				else printimu = true;
				break;
			case CRR:
				Puts("\n\r$");
				break;
    		}
		Toggle_Leds(100);  // Show processor is alive
    	}
}


